# $Id: klass.py,v 1.7 2001/08/10 12:26:00 erik Exp $
# $Source: /cvs/make_product/klass.py,v $
__version__='$Revision: 1.7 $'[11:-2]

"""A bunch of classes for producing klasses (instances that are used
to keep information about the classes they represent)"""

import utils
import string
import os

class basic_klass:
    """A basic klass"""

    def __init__(self, **kw):
        """Initialize an instance of the class"""
        self.id = kw['id']
        self.subklasses = []
        self.methods = []
        self.properties = []
        self.default_indent = '    '
        self.license = ''
        self.meta_type = ''
        self.icon_path = ''
        self.misc_headers = 0
        self.author = ''
        self.author_email = ''
        self.authored_year = ''
        self.type = 'SimpleItem'

    def add_subklass(self, klass):
        """Add a subklass"""
        if not utils.find_entry(self.list_subklasses(), klass.id):
            self.subklasses.append(klass)
        else:
            raise "DuplicateSubklass", "There already exists a " + \
                  "subklass with that 'id'"
        return klass

    def list_subklasses(self):
        """Return a list of all subklasses"""
        return self.subklasses

    def remove_subklass(self, subklass_id):
        """Remove the subklass with the id 'subklass_id'"""
        instance = utils.find_entry(self.list_subklasses(), subklass_id)
        if instance: self.subklasses.remove(instance)

    def get_subklass(self, subklass_id):
        """Return the subklass with id 'subklass_id'"""
        return utils.find_entry(self.list_subklasses(), subklass_id)

class method_klass(basic_klass):
    """A class that provides methods for manipulating methods"""

    def add_method(self, method):
        """Add a method"""
        if not utils.find_entry(self.list_methods(), method.id):
            self.methods.append(method)
        else:
            raise "DuplicateMethod", "There already exsists a " + \
                  "method with that 'id'"
        return method
        
    def list_methods(self):
        """Return a list methods"""
        return self.methods
    
    def remove_method(self, method_id):
        """Remove the method with the id 'method_id'"""
        instance = utils.find_entry(self.list_methods(), method_id)
        if instance: self.methods.remove(instance)
  
    def get_method(self, method_id):
        """Return the method with id 'method_id'"""
        return utils.find_entry(self.list_methods(), method_id)

class method_klass_rendrer(method_klass):
    """Adds methods for rendering"""

    def _render_methods(self, indent=''):
        """Render all methods in this class"""
        methods = ""
        for method in self.list_methods():
            methods = methods + method.render_self(indent) + "\n"
        return methods 

class property_klass(basic_klass):
    """A class that provides methods for manipulating properties"""

    def add_property(self, property):
        """Add a property"""
        if not utils.find_entry(self.list_properties(), property.id):
            self.properties.append(property)
        else:
            raise "DuplicateProperty", "There already exsists a " + \
                  "property with that 'id'"
        return property
        
    def list_properties(self):
        """Return a list of all properties"""
        return self.properties
    
    def remove_property(self, property_id):
        """Remove the property with id 'property_id'"""
        instance = utils.find_entry(self.list_properties(), property_id)
        if instance: self.properties.remove(instance)

    def get_property(self, property_id):
        """Get property with id 'property_id'"""
        return utils.find_entry(self.list_properties(), property_id)
                                    
class property_klass_rendrer(property_klass):
    """Adds methods for rendering"""

    def _render_properties(self, indent=''):
        """Render a list of properties dictionaries"""
        properties = indent + ""
        for property in self.list_properties():
            if properties <> "":
                properties = properties + ",\n"
            properties = properties + indent + self.default_indent + \
                         property.render_self(self.default_indent)
        return properties

    def _render_properties_dictionary(self, indent=''):
        """Render the _properties dictionary"""
        if string.strip(self._render_properties()):
            dictionary = indent + self.default_indent + "_properties = (\n"
            dictionary = dictionary + self._render_properties(indent) + "\n"
            dictionary = dictionary + indent + (2 * self.default_indent) + ")\n"
        else:
            dictionary = indent + self.default_indent + "_properties = ()\n"
        return dictionary

class klass_rendrer(method_klass_rendrer, property_klass_rendrer):
    """Adds methods for rendering"""

    def _render_license(self):
        """Render the license"""
        return self.license % {'author':self.author,
                               'author_email':self.author_email,
                               'authored_year':self.authored_year}

    def _render_name_and_subclasses(self):
        """Render the class name and any subclasses"""
        text = "class " + self.id
        text = text + "("+self.type+",\n"
        text = text + self.default_indent + "base_item.base_item):\n"
        return text

    def _render_type_import_path(self):
        """Return the import path for the 'type'"""
        if self.type == 'SimpleItem':
            return "from OFS.SimpleItem import SimpleItem"
        elif self.type == 'Folder':
            return "from OFS.Folder import Folder"
        else:
            raise "UnknownType", "self.type is bogus"

    def _render_imports(self):
        """Return a string of needed imports"""
        text = self._render_type_import_path() + "\n"
        text = text + "from Globals import DTMLFile, InitializeClass\n"
        text = text + "from AccessControl import ClassSecurityInfo\n"
        text = text + "import base_item\n"
        for subklass in self.list_subklasses():
            text = text + "import " + subklass.id + "\n"
        return text

    def _render_misc_definitions(self):
        """Render misc stuff"""
        text = self.default_indent + "meta_type = '" + self.meta_type + "'\n"
        return text + self.default_indent + "icon = '" + self.icon_path + "'\n"

    def _render_meta_types_and_accessor_calls(self):
        """Render the meta_types dictionary"""
        dict = "{'name': '%(id)s', 'action': '%(id)s_add_form'},"
        if self.list_subklasses():
            text = self.default_indent + "meta_types = (\n"
            for subklass in self.list_subklasses():
                text = text + (2 * self.default_indent) + \
                       dict % {'id': subklass.id} + "\n"
            text = text + (2 * self.default_indent) + ")\n\n"
        else:
            text = ""
        return text

    def _render_accessors(self):
        """Render the accessors for meta_types"""
        str_ = "%(meta_type)s_add_form = %(meta_type)s." + \
               "%(meta_type)s_add_form\n" + self.default_indent + \
               "manage_add_%(meta_type)s = %(meta_type)s." + \
               "manage_add_%(meta_type)s\n"
        text = ""
        for klass_ in self.list_subklasses():
            text = text + self.default_indent + str_ % {'meta_type': klass_.id}

        if text:
            text = text + "\n"
        return text

    def _render_misc_headers(self):
        """Render misc headers"""
        text = ""
        if self.misc_headers:
            text = "# $Id$\n"
            text = text + "# $Source$\n"
            text = text + "__version__ = '$Revision$'[11:-2]\n\n"
            text = text + self.module_docstring + "\n"
        return text

    def _render_add_function(self):
        """Render the def *add() function and its DTMLFile"""
        text = "%(id)s_add_form = DTMLFile('dtml/%(id)s_add_form', globals())\n\n" % \
               {'id': self.id}
        text = text + """def manage_add_%(id)s(self, id, REQUEST=None, **kw):
    \"\"\"Add an instance of the %(id)s class\"\"\"
    self._setObject(id, %(id)s(id, kw))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)\n""" % {'id': self.id}
        return text

    def _render_initialize_class(self):
        """Render the InitializeClass call"""
        return "InitializeClass(%s)" % self.id + "\n"

    def render_self(self):
        """Render the klass instance in text.  This will be
        used when the product class will write itself out to
        the filesystem."""
        text = self._render_license() + "\n"
        misc_headers = self._render_misc_headers()
        if misc_headers:
            text = text + misc_headers + "\n"
        imports = self._render_imports()
        if imports:
            text = text + imports + "\n"
        text = text + self._render_add_function() + "\n"
        text = text + self._render_name_and_subclasses()
        text = text + self.default_indent + self.docstring + "\n\n"
        text = text + self._render_misc_definitions() + "\n"
        text = text + self._render_meta_types_and_accessor_calls()
        text = text + self._render_accessors()
        text = text + self._render_properties_dictionary() + "\n"
        text = text + self._render_set_security_info() + "\n"
        text = text + self._render_methods(self.default_indent)
        text = text + self._render_initialize_class()
        return text

    def _render_for_class_structure(self, indent):
        """Return a line about ourself"""
        return indent + self.id + ' ' + self.meta_type + '\n' + \
               indent + '  ' + self.docstring + '\n'

    def render_class_structure(self, indent):
        """Return a string-representation of this klass and any
        subklasses.  The representation is nicely, and specifically,
        formatted for the render_class_structure() of the
        klass_product_rendrer class."""
        text = self._render_for_class_structure(indent)
        indent = indent + "  "
        for subklass in self.list_subklasses():
            text = text + subklass.render_class_structure(indent)
        return text

    def _render_set_security_info(self):
        """Return the ClassSecurityInfo()-string"""
        return self.default_indent + "security = ClassSecurityInfo()\n"

class klass(method_klass, property_klass, klass_rendrer):
    """A klass"""

    def __init__(self, **kw):
        """Initialize an instance of the class"""
        self.id = kw['id']
        self.methods = []
        self.properties = []
        try: self.default_indent = kw['default_indent']
        except KeyError: self.default_indent = '    '
        try: self.license = kw['license']
        except KeyError: self.license = ''
        try: self.docstring = kw['docstring']
        except KeyError: self.docstring = '"""doc string"""'
        self.subklasses = []
        try: self.meta_type = kw['meta_type']
        except KeyError: self.meta_type = ''
        try: self.icon_path = kw['icon_path']
        except KeyError: self.icon_path = ''
        try: self.misc_headers = kw['misc_headers']
        except KeyError: self.misc_headers = 0
        try: self.module_docstring = kw['module_docstring']
        except KeyError: self.module_docstring = '"""doc string"""'
        try: self.author = kw['author']
        except KeyError: self.author = ''
        try: self.author_email = kw['author_email']
        except KeyError: self.author_email = ''
        try: self.authored_year = kw['authored_year']
        except KeyError: self.authored_year = ''
        try: self.type = kw['type']
        except KeyError: self.type = 'SimpleItem'

    def _setup_default(self):
        """Add default methods and properties"""
        body = self.default_indent + "self.id = id\n\n"
        body = body + self.default_indent + "for property in self._properties:\n"
        body = body + (2 * self.default_indent) + \
               "if property['id'] in values.keys():\n"
        body = body + (3 * self.default_indent) + \
                "self._setPropValue(property['id'], values[property['id']])\n"
        body = body + (2 * self.default_indent) + \
            "elif property.has_key('default'):\n"
        body = body + (3 * self.default_indent) + \
                "self._setPropValue(property['id'], property['default'])\n"

        self.add_method(
            id='__init__', arguments=['self', 'id', ['values', {}]], body=body,
            docstring='"""Initialize an instance of the class"""')

    def _write_subklasses(self, directory):
        """Write all subclasses"""
        for subklass in self.list_subklasses():
            subklass.write_self(directory)

    def _write_add_form(self, directory):
        """Write the add_form.dtml-file"""
        file = open(os.path.join(
            directory, 'dtml', self.id + '_add_form.dtml'), 'w')
        file.write(open(os.path.join(
            'templates', 'add_form.dtml')).read() % {'klass': self.id})
        file.close()
                        
    def write_self(self, directory):
        """Write the rendered klass to a file"""
        file = open(os.path.join(directory, self.id) + '.py', 'w')
        file.write(self.render_self())
        file.close()
        self._write_add_form(directory)
        self._write_subklasses(directory)
