# $Id: product.py,v 1.4 2001/07/30 06:49:06 erik Exp $
# $Source: /cvs/make_product/product.py,v $
__version__='$Revision: 1.4 $'[11:-2]

"""A bunch of classes that mimick some Product behaviour."""

import utils
import os

class basic_product:
    """A basic product"""

    def __init__(self, **kw):
        """Initialize an instance of the class"""
        self.id = kw['id']
        self.author = ''
        self.author_email = ''
        self.authored_year = ''

class klass_product(basic_product):
    """This class provides methods for manipulating classes"""

    def __init__(self, **kw):
        """Initialize an instance of the class"""
        self.id = kw['id']
        self.klasses = []

    def add_klass(self, klass):
        """Add a klass instance"""
        if not utils.find_entry(self.list_klasses(), klass.id):
            self.klasses.append(klass)
        else:
            raise "DuplicateKlass", "There already exists a " + \
                  "klass with that 'id'"
        return klass

    def list_klasses(self):
        """Return a list of all klass instances"""
        return self.klasses

    def remove_klass(self, klass_id):
        """Remove the klass with the id 'klass_id'"""
        instance = utils.find_entry(self.list_klasses(), klass_id)
        if instance: self.klasses.remove(instance)

    def get_klass(self, klass_id):
        """Return the klass instance with id 'klass_id'"""
        return utils.find_entry(self.list_klasses(), klass_id)

class klass_product_rendrer(klass_product):
    """This class provides methods for rendering the klass layout in
    klass_products"""

    def render_class_structure(self):
        """Return a nice string-representation of the class
        structure"""
        text = self.id + "\n"
        text = "  " + self.docstring + "\n"
        indent = "  "
        for klass in self.list_klasses():
            text = text + klass.render_class_structure(indent)
        return text

class zope_product(klass_product, klass_product_rendrer):
    """A class that provides methods for setting up a Zope Python
    Product"""

    def __init__(self, **kw):
        """Initialize an instance of the class"""
        self.id = kw['id']
        try: self.license = kw['license']
        except KeyError: self.license = ''
        try: self.docstring = kw['docstring']
        except KeyError: self.docstring = '"""doc string"""'
        try: self.default_indent = kw['default_indent']
        except KeyError: self.default_indent = '    '
        self.klasses = []
        try: self.directory = os.path.join(kw['directory'], self.id)
        except KeyError: self.directory = self.id
        try: self.author = kw['author']
        except KeyError: self.author = ''
        try: self.author_email = kw['author_email']
        except KeyError: self.author_email = ''
        try: self.authored_year = kw['authored_year']
        except KeyError: self.authored_year = ''
        try: self.form_creator = kw['form_creator']
        except KeyError: self.form_creator = 'self'
        
    def _add___init__py(self):
        """Add __init__.py file to the product directory"""
        if self.list_klasses():
            klass = self.list_klasses()[0].id
        else:
            klass = 'None'
        module = klass
        file = open(os.path.join(self.directory, '__init__.py'), 'w')
        file.write(open(os.path.join('templates', '__init__.py')).read() % \
                   {'license': self.license, 'id': self.id,
                    'klass': klass, 'module': module})
        file.close()

    def _add_versiontxt(self):
        """Add version.txt file to the product directory"""
        file = open(os.path.join(self.directory, 'version.txt'), 'w')
        file.write('0.0.0')
        file.close()

    def _add_help_directory(self):
        """Add help/ directory"""
        os.mkdir(os.path.join(self.directory, 'help'))

    def _add_www_directory(self):
        """Add the www/ directory"""
        os.mkdir(os.path.join(self.directory, 'www'))

    def _add_dtml_directory(self):
        """Add the dtml/ directory"""
        os.mkdir(os.path.join(self.directory, 'dtml'))

    def _create_directory(self):
        """Create the directory in Products/"""
        if os.path.isdir(self.directory):
            raise "DirectoryExists", "The product directory " + \
                  self.directory + " already exist.  Please remove first."
        else:
            os.mkdir(self.directory)

    def _add_base_item_py(self):
        """Add the base_item.py file"""
        file = open(os.path.join(self.directory, 'base_item.py'), 'w')
        file.write(open(os.path.join('templates', 'base_item.py')).read())
        file.close()

    def _setup_defaults(self):
        """Set up some defaults"""
        self._create_directory()
        self._add___init__py()
        self._add_versiontxt()
        self._add_help_directory()
        self._add_www_directory()
        self._add_dtml_directory()
        self._add_base_item_py()
            
    def _write_klasses(self):
        """Write all klasses to files in self.directory"""
        for klass in self.list_klasses():
            klass.write_self(self.directory)

    def write_self(self):
        """Write the product to the filesystem"""
        self._setup_defaults()
        self._write_klasses()
